/**
 *  滑动控件
 *
 * init(element, [callback])    初始化方法
 * 参数：
 *     element：需要滑动的DIV容器，里面的每一个 item 都
 *     callback({index})：回调方法
 *
 * trigger(target)              触发滑动动作
 * 参数：
 *    target：需要显示 item 的索引
 */
(function ($, root, factory) {
    root['swipeSlide'] = new factory($, root);
})(window.jQuery, window, function ($, root) {

    var swipe = function swipe() {
        this.element = null;
        // 回调方法
        this.callbackFunc = null;

        this.stage = null;
        this._items = [];

        this.stageWidth = 0;
        this.itemWidth = document.documentElement.clientWidth;

        this.eventsName = ['touchstart', 'touchmove', 'touchend'];

        this.drag = {};
        this.drag.target = 0; // 当前显示 item 的索引
        this.drag.pull = 30; // 每次滑动拉取多距离时，会切换到下一个 item
        this.drag.current = { x: 0, y: 0 };
        this.drag.touchstart = this.drag.current;
        this.drag.touchmove = this.drag.current;

        this.options = {
            loadingClass: 'owl-loading',
            loadedClass: 'owl-loaded',
            itemClass: 'owl-item'
        };

        this.settings = {
            itemElement: 'div',
            stageElement: 'div',

            stageClass: 'owl-stage',
            stageOuterClass: 'owl-stage-outer',

            nestedItemSelector: false
        };
    };

    /**
     * Prepares an item before add.
     * @todo Rename event parameter `content` to `item`.
     * @protected
     * @returns {jQuery|HTMLElement} - The item container.
     */
    swipe.prototype.prepare = function (item) {
        var data = $('<' + this.settings.itemElement + '/>').addClass(this.options.itemClass).append(item);
        return data;
    };

    /**
     * Replaces the current content.
     * @public
     * @param {HTMLElement|jQuery|String} content - The new content.
     */
    swipe.prototype.replace = function (content) {
        this.stage.empty();
        this._items = [];

        if (content) {
            content = content instanceof jQuery ? content : $(content);
        }

        content.filter(function () {
            return this.nodeType === 1;
        }).each($.proxy(function (index, item) {
            item = this.prepare(item);
            this.stage.append(item);
            this._items.push(item);
            this.stageWidth = this.stageWidth + this.itemWidth;
        }, this));
    };

    swipe.prototype.transform = function (x, time) {
        return {
            'transform': 'translate3d(' + x + 'px, 0px, 0px)',
            'transition': time + 's',
            'width': this.stageWidth
        };
    };

    swipe.prototype.slideEnd = function () {
        var x = this.drag.touchmove.x - this.drag.touchstart.x;

        if (this.drag.target > 0 && x > this.drag.pull) {
            // 从左向右滑动
            this.trigger(this.drag.target - 1);
        } else if (this.drag.target < this._items.length - 1 && x < -this.drag.pull) {
            // 从右向左滑动
            this.trigger(this.drag.target + 1);
        } else {
            // 不需要切换页，但需要还原位置
            this.trigger(this.drag.target);
        }
    };

    swipe.prototype.getCallbackFuncParams = function () {
        return {
            index: this.drag.target
        };
    };

    /**
     * Create the stage DOM element
     */
    swipe.prototype.initializeStage = function () {
        this.stage = this.element.find('.' + this.settings.stageClass);

        // if the stage is already in the DOM, grab it and skip stage initialization
        if (this.stage.length) {
            return;
        }

        this.element.addClass(this.options.loadingClass);

        // create stage
        this.stage = $('<' + this.settings.stageElement + '>', {
            "class": this.settings.stageClass
        }).wrap($('<div/>', {
            "class": this.settings.stageOuterClass
        }));

        // append stage
        this.element.append(this.stage.parent());
    };

    /**
     * Gets unified pointer coordinates from event.
     * @todo #261
     * @protected
     * @param {Event} - The `mousedown` or `touchstart` event.
     * @returns {Object} - Contains `x` and `y` coordinates of current pointer position.
     */
    swipe.prototype.pointer = function (event) {
        var result = { x: null, y: null };

        event = event.originalEvent || event || window.event;

        event = event.touches && event.touches.length ? event.touches[0] : event.changedTouches && event.changedTouches.length ? event.changedTouches[0] : event;

        if (event.pageX) {
            result.x = event.pageX;
            result.y = event.pageY;
        } else {
            result.x = event.clientX;
            result.y = event.clientY;
        }

        return result;
    };

    /**
     * Create item DOM elements
     */
    swipe.prototype.initializeItems = function () {
        var $items = this.element.find('.owl-item');

        // if the items are already in the DOM, grab them and skip item initialization
        if ($items.length) {
            return;
        }

        // append content
        this.replace(this.element.children().not(this.stage.parent()));

        this.element.removeClass(this.options.loadingClass).addClass(this.options.loadedClass);
    };

    swipe.prototype.initializeWidth = function () {
        $(this._items).each(function (index, item) {
            item.css({
                'width': this.itemWidth,
                'float': 'left'
            });
        }.bind(this));

        $(this.stage).css(this.transform(0, 0.25));
    };

    swipe.prototype.bindEventCallback = function (typeName, event) {
        var pointer = this.pointer(event);
        switch (typeName) {
            case this.eventsName[0]:
                this.drag.touchstart = this.drag.touchmove = pointer;
                break;
            case this.eventsName[1]:
                this.drag.touchmove = pointer;
                var x = this.drag.current.x + (this.drag.touchmove.x - this.drag.touchstart.x);
                $(this.stage).css(this.transform(x, 0));
                break;
            case this.eventsName[2]:
                this.slideEnd();
                if (this.callbackFunc) {
                    this.callbackFunc(this.getCallbackFuncParams());
                }
                break;
        }
    };

    swipe.prototype.initializeBindEvent = function () {
        $.each(this.eventsName, function (index, eventName) {
            this.element.on(eventName, this.bindEventCallback.bind(this, eventName));
        }.bind(this));
    };

    swipe.prototype.init = function (element, callbackFunc) {
        this.element = element;
        this.callbackFunc = callbackFunc;
        this.initializeStage();
        this.initializeItems();
        this.initializeWidth();
        this.initializeBindEvent();
    };

    swipe.prototype.trigger = function (target) {
        this.drag.target = target;
        this.drag.current.x = this.drag.target * -this.itemWidth;
        $(this.stage).css(this.transform(this.drag.current.x, 0.25));
    };

    return new swipe();
});